   --------------------------------------------------------------------
--|  Program: Hurkle                           Version : 1.0b
   --------------------------------------------------------------------
--|  Abstract : Plays the game of "Find the Hurkle."
--|             This program simulates a simple guessing game in which 
--|             the object is to guess the location of the Hurkle on a 
--|             9x9 grid.  The program will print the grid to the      
--|		screen, randomly generate a hiding place for the Hurkle
--|		allow the user 5 guesses, update the grid to show      
--|		locations guessed and give hints as to where to guess  
--| 		next. 
--|             Refer to the specification for rules of play.
   --------------------------------------------------------------------
--|  File            : hurkle.adb
--|  Compiler/System : IBM AIX/6000 Ada
--|  Author          : John Dalbey             Date : 4/30/94
--|
   --------------------------------------------------------------------
--|  Notes           :
--|
--|  Version History : Version 1 is just a skeleton with stubs for almost
--|                    all the procedures.  This is the source code for 
--|                    the lab activity.
   --------------------------------------------------------------------
WITH Text_IO; 
USE  Text_IO;

PROCEDURE Hurkle IS

--
--     Subtype and constant declarations
--
SUBTYPE Row IS CHARACTER RANGE 'A'..'I';  -- Defines legal rows on the grid
SUBTYPE Col IS INTEGER   RANGE  1 .. 9;   -- Defines legal columns on the grid

MaxTurns    : CONSTANT INTEGER := 5;       -- Number of turns player allowed

PACKAGE My_Int_IO IS NEW INTEGER_IO (INTEGER);



   --------------------------------------------------------------------
   PROCEDURE Display_Grid IS
   --
   --  Purpose: Display instructions for the game and a diagram of the
   --           hurkle grid.
   --  Called by: Main
   --  Assumes: nothing.
   --  Returns: nothing.
   --
   BEGIN
      PUT_LINE( "Stub for DisplayGrid" );
   END Display_Grid;



   --------------------------------------------------------------------
   PROCEDURE Hide_Hurkle(RowSpot:  OUT Row;
                         ColSpot:  OUT Col) IS
   --
   -- Purpose:  Pick a random hiding location on the grid.
   -- Called by: Main
   -- Assumes: nothing.
   -- Returns: Random values for RowSpot and ColSpot.
   --
   BEGIN
      PUT_LINE( "Stub for HideHurkle" );
      -- For the stub, just "hard code" the hiding spot to E5.
      -- This will need to be changed later.
      RowSpot := 'E';
      ColSpot := 5;
   END Hide_Hurkle;



   --------------------------------------------------------------------
   FUNCTION Out_of_Turns (Turn: INTEGER) RETURN BOOLEAN IS
   --
   -- Purpose:  Determine if player has used up all their turns.
   -- Called by: Main
   -- Assumes: Turn has been initialized.
   -- Returns: TRUE if number of turns is greater than max allowed.
   --          FALSE otherwise.
   --
   BEGIN      
      RETURN Turn >= MaxTurns;
   END Out_of_Turns;



   --------------------------------------------------------------------
   FUNCTION Found_Spot (RowSpot, RowGuess: IN Row;
                        ColSpot, ColGuess: IN Col)  RETURN BOOLEAN IS
   --
   -- Purpose:  Determine if player guessed the hurkle's hiding spot.
   -- Called by: Main
   -- Assumes: HideHurkle and GetValid guess have been called (so that
   --          hiding spot and guess have legal values.
   -- Returns: TRUE if guess equals hiding spot, FALSE otherwise.
   --
   BEGIN
      PUT_LINE( "Stub for FoundSpot" );
      RETURN FALSE;
   END Found_Spot;



   --------------------------------------------------------------------
   PROCEDURE Get_Valid_Guess(RowGuess: OUT Row;
                             ColGuess: OUT Col) IS
   --
   -- Purpose:  Obtain from the user a guess for the row and column.
   --           If an exception occurs for out of range input data,
   --           trap the error and allow the user to re-enter the data.
   -- Called by: Main
   -- Assumes: nothing
   -- Returns: legal values for row and column guess.
   --
   BEGIN
      PUT_LINE( "Stub for GetValidGuess" );
      Text_IO.GET( RowGuess );
      My_Int_IO.GET( ColGuess );
   END Get_Valid_Guess;



   --------------------------------------------------------------------
   PROCEDURE Give_Hint(RowSpot, RowGuess: IN Row;
                       ColSpot, ColGuess: IN Col) IS
   --
   -- Purpose:    Analyze the player's guess and display a hint.
   --    The hint indicates which compass direction they need to look.
   --    Their are 8 possible messages:  Go North, Go South, Go East, Go West
   --        Go NorthWest, Go NorthEast, Go SouthWest, Go SouthEast
   --    If the player has guessed the exact hiding spot, no hint is given.
   -- Called by: Main
   -- Assumes: HideHurkle and GetValid guess have been called (so that
   --          hiding spot and guess have legal values.
   -- Returns: nothing.
   --
   BEGIN
      PUT_LINE( "Stub for GiveHint" );
   END Give_Hint;



   --------------------------------------------------------------------
   PROCEDURE Display_Summary( Turn             : IN INTEGER;
                              RowSpot, RowGuess: IN Row;
                              ColSpot, ColGuess: IN Col)  IS
   --
   -- Purpose:  Display a message indicating how the game ended, specifically:
   --   If they correctly guessed the hurkle's hiding spot,
   --       display a message that the player found the hurkle
   --       and how many turns it took.
   --   If they ran out of turns display a message to that effect.
   -- Called by: Main
   -- Assumes: HideHurkle and GetValid guess have been called (so that
   --          hiding spot and guess have legal values.
   --          Turn has been initialized.
   -- Returns: nothing.
   --
   BEGIN
      PUT_LINE( "Stub for DisplaySummary" );
      PUT_LINE( "Sorry, you ran out of turns." );
   END Display_Summary;



--------------------------------------------------------------------
PROCEDURE Main IS

Turn        : INTEGER := 0;       -- Count how many turns so far

RowHideSpot : Row;                -- Row on which hurkle hides
ColHideSpot : Col;                -- Col on which hurkle hides

GuessRow    : Row;                -- User's guess for row
GuessCol    : Col;                -- User's guess for col

BEGIN          -- The main logic of the game is here

   -- First display instructions and a picture of the grid, then
   -- pick a random hiding spot for the hurkle.

   Display_Grid;
   Hide_Hurkle( RowHideSpot, ColHideSpot );


   -- Repeat entering a guess and giving hints until they run out of
   -- turns or guess correctly.

   LOOP 

      Get_Valid_Guess( GuessRow, GuessCol );
      Give_Hint( RowHideSpot, GuessRow, ColHideSpot, GuessCol );
      Turn := Turn + 1;    -- One more turn has passed

      EXIT WHEN Out_Of_Turns( Turn ) OR
                Found_Spot( RowHideSpot, GuessRow, ColHideSpot, GuessCol );

   END LOOP;

   -- Print a concluding message

   Display_Summary( Turn, RowHideSpot, GuessRow, ColHideSpot, GuessCol );

END Main;


BEGIN
    Main;
END Hurkle;
