   --------------------------------------------------------------------
--|  Program         : Inventory Update              Version : 1.0 
   --------------------------------------------------------------------
--|  Abstract        : Menu driven inventory analysis program.
--|                  : See specification document for details.
--|                  :
--|  Compiler/System : IBM AIX/6000 Ada
--|  Author          : John Dalbey             Date : 4/30/94
   --------------------------------------------------------------------
--|  Notes           :
--|
--|  Version History : V1.0 is a skeleton with minimal functionality.
--|                  : This is the source code for the lab activity.
--|
   --------------------------------------------------------------------
WITH Text_IO; 
USE  Text_IO;

PROCEDURE InvUpdat IS

--
--  Declarations of constants and types
--
Stringsize : constant INTEGER := 20;            -- max string length
MaxStock   : constant INTEGER := 200;           -- max # array size

SUBTYPE String20 is STRING(1..Stringsize);

TYPE Item is       -- an inventory item
   RECORD                             
      CatalogCode  : String(1..10);   -- code identifying the item
      Dept         : CHARACTER;       -- department code
      Quantity     : NATURAL;         -- current quantity on hand
      PartName     : String20;        -- description of item
   END RECORD;

TYPE Inventype is ARRAY (1..MaxStock) OF Item; -- the array type

PACKAGE My_Int_IO IS NEW INTEGER_IO (Natural);
PACKAGE My_Flt_IO IS NEW FLOAT_IO (FLOAT);

---------------------------------------------------------------------
PROCEDURE Load_File( Inventory:    OUT Inventype;
                     RecCount : IN OUT NATURAL;
                     FileName : IN OUT STRING) IS

-- Purpose: Load the data from an external file into an array.
-- ASSUMES: theFile has at least one record.
--          FileName contains the name of the external file.
-- RESULTS: Inventory contains records from theFile.
--          RecCount has number of records.

Length  : Natural;              -- input string length
theFile : Text_IO.File_Type;    -- identify the file
Dummy   : Character;            -- dummy input character

BEGIN

    -- Open the data file for input
    Text_IO.Open( theFile, In_File, FileName(1..Filename'Last));
    RecCount := 0;

    --  Read all records from the file into array Inventory
    WHILE NOT End_of_File(theFile) LOOP

         RecCount := RecCount + 1;    -- increment record count

         -- Read the data for one record
         Get (theFile, Inventory(RecCount).CatalogCode);
         Get (theFile, Inventory(RecCount).Dept );
         My_Int_IO.Get (theFile, Inventory(RecCount).Quantity );
         Get (theFile, Dummy);
         Get_Line (theFile, Inventory(RecCount).PartName,Length);

    END LOOP;

    Text_IO.Close(theFile);

    -- Display the number of items read
    My_Int_IO.Put (RecCount, WIDTH=>5);
    Put (" records read from ");
    Put_Line (FileName(1..Filename'Last));

END Load_File;

---------------------------------------------------------------------
PROCEDURE Print_Record (AnItem: IN Item) IS
--
-- Purpose: Display a single inventory record.
-- Assumes: AnItem has been assigned values.
-- Returns: nothing.
--
BEGIN
         Put (AnItem.CatalogCode);
         Put ("      ");
         Put (AnItem.Dept);
         Put ("   ");
         Put (AnItem.PartName);
         Put ("   ");
         My_Int_IO.Put (AnItem.Quantity, WIDTH=>5 );
         Put ("   ");
         New_Line;

END Print_Record;
         

---------------------------------------------------------------------
PROCEDURE Get_Dept_Code (dept_code: OUT CHARACTER) IS
--
-- Purpose: Prompt the user and obtain a department code.
-- Assumes: nothing.
-- Returns: a single character read from the user.
--
BEGIN

    Text_IO.Put ("Enter Department code (A,B,C,D, or * = all) : ");
    Text_IO.Get (Dept_code);
    Text_IO.Skip_Line;

END Get_Dept_Code;

---------------------------------------------------------------------
PROCEDURE Print_Headers IS
--
-- Purpose: Display column headers for inventory list.
-- Assumes: nothing.
-- Returns: nothing.
--
BEGIN
   Text_IO.Put("Catalog Code  Dept   Part Name         ");
   Text_IO.Put(" Quantity  ");
   Text_IO.New_Line;
END Print_Headers;
   
---------------------------------------------------------------------
PROCEDURE Print_Inventory(Inventory: Inventype;
                          RecCount : NATURAL) IS
--
-- Purpose: Display all records of the inventory.
-- ASSUMES: RecCount has a value.
-- Returns: nothing.
--
Code  : CHARACTER; -- department code chosen by user
BEGIN
    -- Obtain dept to be printed
    Get_Dept_Code (Code);

    -- Display the column headers
    Print_Headers;

    -- Examine every record in the array
    FOR Index in 1 .. RecCount LOOP

        -- Select only records from the desired department
        -- (Add code here)
        Print_Record (Inventory (Index) );   -- display a record
                
    END LOOP;

END Print_Inventory;

---------------------------------------------------------------------
PROCEDURE Print_Totals(Inventory: Inventype;
                       RecCount : NATURAL) IS
--
-- Purpose: Compute and Display total value of merchandise.
-- ASSUMES: RecCount has a value.
-- Returns: nothing.
--
Code  : CHARACTER;          -- department code chosen by user
TotalValue : FLOAT := 0.0;  -- Sum of merchandise values

BEGIN

    -- obtain dept to be totaled
    Get_Dept_Code (Code);
 
    -- Examine every record in the array
    FOR Index in 1 .. RecCount LOOP

        -- Select only records from the desired department
        -- (Add code here)

           --  Compute quantity on hand value and add to total
           -- (Add code here)
           NULL;

    END LOOP;

    -- Display the total
    Text_IO.New_Line;
    Text_IO.Put ("The total value is $");
    My_Flt_IO.Put (TotalValue, FORE=>5, AFT=>2, EXP=>0);
    Text_IO.New_Line;

END Print_Totals;


---------------------------------------------------------------------
PROCEDURE Display_Menu(FileName:String20) IS
--
--  Purpose: Displays list of program options for the user
--  Assumes: FileName has been initialized (may be blank)
--  Returns: nothing..
--
BEGIN
   Text_IO.Put (ASCII.ESC & "[2J");         -- clear screen
   Text_IO.New_Line;
   Text_IO.Set_Col(30);
   Text_IO.Put_Line ("Inventory Analysis Program");
   Text_IO.New_Line;
   Text_IO.Set_Col(30);
   Text_IO.Put ("Data File : ");
   Text_IO.Put_Line (FileName(1..20));    -- show current filename
   Text_IO.New_Line;
   Text_IO.Set_Col(35);
   Text_IO.Put_Line ("MENU OPTIONS");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("1)  Load the inventory data from Data File.");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("2)  Print a list of inventory data.");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("3)  Print total value of merchandise.");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("4)  Find quantity on hand for an item. ");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("5)  Select different Data File.");
   Text_IO.Set_Col(20);
   Text_IO.Put_Line ("6)  Exit");
END Display_Menu;

---------------------------------------------------------------------
PROCEDURE Get_Choice (Option: OUT CHARACTER) IS
--
-- Purpose: Display prompt and obtain the user's choice from the menu.
-- Assumes: The menu has been displayed.
--          Menu options are numbered from 1 to 6.  
-- Returns: A valid menu choice.
--
Reply: CHARACTER := ' ';   -- character read from user
BEGIN
   -- Prompt and obtain response
   Text_IO.Set_Col(20);
   Text_IO.Put ("Enter menu choice (1-6): ");
   Text_IO.Get (Reply);

   -- Verify response is in proper range
   WHILE Reply < '1' OR Reply > '6' LOOP
      Text_IO.Put ("Please choose an option between 1 and 6:");
      Text_IO.Get (Reply);
   END LOOP;

   Text_IO.New_Line;
   Text_IO.Skip_Line;
   Option := Reply;     -- return Reply

END Get_Choice;


---------------------------------------------------------------------
FUNCTION Find_Code (Inventory : IN Inventype;
                    RecCount  : IN NATURAL) RETURN NATURAL IS
--
-- Purpose: Obtain desired catalog code from user.
--          Search the array for desired item.
-- Assumes: RecCount is number of records in array.
-- Returns: The index of the item found, or zero if not found.
--
BEGIN
   Text_IO.Put_Line ("find code not implemented yet.");
   RETURN 0;
END Find_Code;

---------------------------------------------------------------------
FUNCTION Find_Part (Inventory : IN Inventype;
                    RecCount  : IN NATURAL) RETURN NATURAL IS
--
-- Purpose: Obtain desired part name (description) from user.
--          Search the array for desired item.
-- Assumes: RecCount is number of records in array.
-- Returns: The index of the item found, or zero if not found.
--
BEGIN
   Text_IO.Put_Line ("find part not implemented yet.");
   RETURN 0;
END Find_Part;
   
---------------------------------------------------------------------
FUNCTION Search (Inventory : IN Inventype;
                 RecCount  : IN NATURAL) RETURN NATURAL IS
--
--  Purpose: Allow user to enter their search criteria.
--           Lookup the item in the table.
--           Print the item (if it was found) or a warning message (if not).
--  Assumes: nothing
--  Returns: The index of the item found, or zero if not found.
--
Criteria: CHARACTER; -- search preference
Index   : NATURAL;   -- position in array of item found
BEGIN
   -- Prompt and obtain choice
   Text_IO.Put ("Search by Catalog code or by Item Description (C,D)? ");
   Text_IO.Get (Criteria);
   Text_IO.Skip_Line;

   -- Invoke appropriate lookup procedure
   CASE Criteria IS
      WHEN 'C' | 'c' => Index := Find_Code (Inventory, RecCount);
      WHEN 'D' | 'd' => Index := Find_Part (Inventory, RecCount);
      WHEN others => null;
   END CASE;

   -- (Add code here)
   
   RETURN Index;  -- send back the results of the search
END Search;

---------------------------------------------------------------------
PROCEDURE Get_Name (FileName: OUT String20; Length: OUT NATURAL) IS
--
--  Purpose: A prompt is displayed and the filename (for inventory file)
--           is read from the keyboard.
--  Assumes: nothing
--  Results: FileName contains the filename, followed by blanks.
--           Length contains the number of characters in the name.
--
BEGIN
   NULL;
END Get_Name;

---------------------------------------------------------------------
PROCEDURE Continue IS
--
-- Purpose: Wait for user to press the Enter key before proceeding
-- Assumes: nothing.
-- Results: Execution is paused until the user presses Enter.
--
Reply: character;
BEGIN
   Text_IO.New_Line;
   Text_IO.Put ("                      Press <Enter> to continue.");
   WHILE NOT Text_IO.End_of_line LOOP
         Get (Reply);
   END LOOP;
END Continue;

-------------------------  M A I N  -----------------------------------
PROCEDURE Main IS
Inventory: Inventype;                   -- The array of items
RecCount : NATURAL := 0;                -- Number of items in file
FileName : STRING20 := (others => ' '); -- the data file name
choice   : CHARACTER := ' ';            -- user's menu choice
Length   : NATURAL   := 11;             -- file name length
ItemIndex: NATURAL;                     -- Position of found item
BEGIN

   FileName(1..11) := "equip94.dat";   -- Default file name
   Display_Menu (FileName);            -- List user's choices
   Get_Choice (choice);                -- Obtain a choice

   -- Loop until user quits
   WHILE choice /= '6' LOOP           

      -- Invoke appropriate subprogram for user's choice
      CASE choice IS
         WHEN '1' =>  Load_File (Inventory, RecCount, FileName(1..Length));
         WHEN '2' =>  Print_Inventory (Inventory, RecCount);
         WHEN '3' =>  Print_Totals(Inventory, RecCount);
         WHEN '4' =>  ItemIndex := Search(Inventory, RecCount);
         WHEN '5' =>  Get_Name(FileName,Length);
         WHEN '6' =>  null;
         WHEN OTHERS =>
            Text_IO.Put_Line ("Invalid selection in Main.");
      END CASE;

      -- Get the next choice from the user
      Continue;
      Display_Menu (FileName);
      Get_Choice (choice);
   END LOOP;

END Main;

BEGIN
   Main;
END InvUpdat;

